@@ -1,19 +1,20 @@
 #!/usr/bin/env python3
 
-import boxx
 import cv2
+import boxx
 import numpy as np
 
 
 def R_t_to_T(R, t):
     T = np.zeros((4, 4))
     T[:3, :3] = R
-    T[:3, -1] = t
+    T[:3, -1] = t.squeeze()
     T[3, 3] = 1
     return T
 
 
 def r_t_to_T(r, t):
+    assert r.size == 3
     return R_t_to_T(cv2.Rodrigues(r.squeeze())[0], t.squeeze())
 
 
@@ -60,6 +61,7 @@ def point_cloud_to_depth(points, K, xy):
     xyzs[:, :2] /= xyzs[:, 2:]
 
     xyzs = np.array(sorted(xyzs, key=lambda xyz: -xyz[2]))
+    xyzs[:, :2] = xyzs[:, :2].round(0)
     mask = (
         (xyzs[:, 0] >= 0)
         & (xyzs[:, 0] < xy[0])
@@ -69,7 +71,7 @@ def point_cloud_to_depth(points, K, xy):
     xyzs = xyzs[mask, :]
 
     depth = np.zeros(xy[::-1])
-    depth[np.int32(xyzs[:, 1].round(0)), np.int32(xyzs[:, 0].round(0))] = xyzs[:, 2]
+    depth[np.int32(xyzs[:, 1]), np.int32(xyzs[:, 0])] = xyzs[:, 2]
     return depth
 
 
@@ -105,5 +107,57 @@ def intrinsic_format_conversion(K_or_dic):
         return dic
 
 
+def _to_3x_uint8(arr):
+    if arr.dtype != np.uint8:
+        arr = boxx.uint8(boxx.norma(arr))
+        arr = np.uint8(cv2.applyColorMap(arr, cv2.COLORMAP_JET) * 0.75)
+    elif arr.ndim == 2:
+        arr = np.transpose([arr] * 3, (1, 2, 0))
+    return arr
+
+
+def vis_stereo(img1, img2, n_line=21, thickness=0.03):
+    """
+    Draw lines on stereo image pairs, two cases of horizontal rectify and vertical rectify.
+    Input: image numpy pairs.
+    """
+    img1 = _to_3x_uint8(img1)
+    img2 = _to_3x_uint8(img2)
+    colors = [
+        [255, 0, 0],
+        [0, 255, 255],
+        [0, 255, 0],
+        [255, 0, 255],
+        [0, 0, 255],
+        [255, 255, 0],
+    ]
+    vis = np.concatenate([img1, img2], 1)
+    img_size = img1.shape[0]
+    _thickness = max(1, int(round(thickness * img_size / (n_line + 1))))
+    gap = img_size / (n_line + 1)
+    for i in range(n_line):
+        b = int((i + 1) * gap - _thickness / 2)
+        vis[b : b + _thickness] = colors[i % len(colors)]
+    return vis
+
+
+def vis_align(img1, img2, n_line=21, shows=True):
+    img1 = _to_3x_uint8(img1)
+    img2 = _to_3x_uint8(img2)
+    y, x = img1.shape[:2]
+    visv = vis_stereo(img1, img2, n_line=n_line)
+    vis = np.concatenate((visv, vis_stereo(img2, img1, n_line=n_line),), 0)
+    vis = np.rot90(
+        vis_stereo(
+            np.rot90(vis[:y]), np.rot90(vis[y:]), n_line=int(n_line * x * 2 / y)
+        ),
+        3,
+    )
+    viss = [vis[:y, :x], vis[:y, x:], vis[y:, :x], vis[y:, x:]]
+    if shows:
+        boxx.shows(viss)
+    return viss
+
+
 if __name__ == "__main__":
     pass
